import asyncio
from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup
from telegram.ext import ApplicationBuilder, CommandHandler, MessageHandler, CallbackContext, filters, CallbackQueryHandler
import mysql.connector
from datetime import datetime
import os

# تنظیمات دیتابیس
DB_CONFIG = {
    'host': 'localhost',
    'user': 'gtalkir_filetolink',
    'password': 'N{+rO_6Aba3q',
    'database': 'gtalkir_filetolink'
}

# آی‌دی ادمین
ADMIN_ID = 6978344483  # آیدی شما

def get_db_connection():
    return mysql.connector.connect(**DB_CONFIG)

def generate_unique_filename(extension):
    if not os.path.exists('uploads'):
        os.makedirs('uploads')
    count = len(os.listdir('uploads'))
    return f"file-{count + 1}.{extension}"

async def start(update: Update, context: CallbackContext) -> None:
    user_id = update.message.from_user.id
    if not is_user_allowed(user_id):
        await update.message.reply_text('شما اجازه استفاده از این ربات را ندارید. لطفاً با ادمین تماس بگیرید.')
        return

    await update.message.reply_text('به ربات خوش آمدید! لطفاً یک فایل ارسال کنید.')

def is_user_allowed(user_id):
    conn = get_db_connection()
    cursor = conn.cursor()
    cursor.execute("SELECT * FROM users WHERE telegram_id = %s", (user_id,))
    result = cursor.fetchone()
    conn.close()
    return result is not None

async def handle_file(update: Update, context: CallbackContext) -> None:
    user_id = update.message.from_user.id
    if not is_user_allowed(user_id):
        await update.message.reply_text('شما اجازه استفاده از این ربات را ندارید. لطفاً با ادمین تماس بگیرید.')
        return

    file = update.message.document or update.message.audio or update.message.video
    if file is None:
        await update.message.reply_text('لطفاً یک فایل معتبر ارسال کنید.')
        return

    extension = file.file_name.split('.')[-1] if file.file_name and '.' in file.file_name else None

    # پرسش فرمت در صورت نبودن یا نامشخص بودن آن
    if extension is None:
        keyboard = [
            [InlineKeyboardButton("MP3", callback_data="mp3")],
            [InlineKeyboardButton("MP4", callback_data="mp4")],
        ]
        reply_markup = InlineKeyboardMarkup(keyboard)
        context.user_data['file_id'] = file.file_id
        await update.message.reply_text('لطفاً فرمت فایل را انتخاب کنید:', reply_markup=reply_markup)
        return

    await save_and_send_file(update, context, file.file_id, extension)

async def button(update: Update, context: CallbackContext) -> None:
    query = update.callback_query
    await query.answer()
    file_id = context.user_data['file_id']
    extension = query.data
    await save_and_send_file(update, context, file_id, extension)

async def save_and_send_file(update: Update, context: CallbackContext, file_id, extension):
    unique_filename = generate_unique_filename(extension)

    # دریافت و ذخیره فایل
    file_info = await context.bot.get_file(file_id)
    file_path = f"uploads/{unique_filename}"
    try:
        await file_info.download_to_drive(file_path)
    except Exception as e:
        await update.message.reply_text(f'خطا در دانلود فایل: {str(e)}')
        return

    # آیدی کاربر را به طور پویا از update.message یا update.callback_query بگیرید
    user_id = update.message.from_user.id if update.message else update.callback_query.from_user.id
    save_file_to_db(user_id, file_path)

    # ساخت لینک دانلود و ارسال پیام
    download_url = f"http://gtalk.ir/{file_path}"
    if update.message:
        await update.message.reply_text(f"لینک دانلود فایل شما:\n{download_url}")
    else:
        await update.callback_query.edit_message_text(f"لینک دانلود فایل شما:\n{download_url}")

def save_file_to_db(user_id, file_path):
    conn = get_db_connection()
    cursor = conn.cursor()
    cursor.execute("INSERT INTO files (user_id, file_path, upload_date) VALUES (%s, %s, %s)",
                   (user_id, file_path, datetime.now()))
    conn.commit()
    conn.close()

async def admin_panel(update: Update, context: CallbackContext):
    user_id = update.message.from_user.id
    if user_id != ADMIN_ID:
        await update.message.reply_text('شما دسترسی به این بخش را ندارید.')
        return

    conn = get_db_connection()
    cursor = conn.cursor()
    cursor.execute("SELECT COUNT(*) FROM users")
    users_count = cursor.fetchone()[0]

    cursor.execute("SELECT SUM(LENGTH(file_path)) FROM files")
    used_space = cursor.fetchone()[0] or 0

    await update.message.reply_text(f"پنل مدیریتی:\n- تعداد کاربران: {users_count}\n- فضای استفاده شده: {used_space / (1024 * 1024):.2f} مگابایت")
    conn.close()

def main():
    application = ApplicationBuilder().token("7130566272:AAHoCpH6CmYNR6PFjJUonVrRiu7r6xb3iqg").build()

    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("admin", admin_panel))
    application.add_handler(MessageHandler(filters.Document.ALL | filters.AUDIO | filters.VIDEO, handle_file))
    application.add_handler(CallbackQueryHandler(button))

    print("ربات در حال اجراست...")
    application.run_polling(allowed_updates=Update.ALL_TYPES)

if __name__ == '__main__':
    main()
